# Generic Makefile components for a LaTeX paper
# Last edited on 2015-11-05 03:16:08 by stolfilocal

########################################################################
# Main section
# Client must define ${JOBNAME} ${FORMAT}
# 
# The ${FORMAT} should be "ps" to use plain latex for dvi -> Postscript -> PDF,
# or "pdf" to use pdflatex and generate PDF directly.
# 
# The client may also define
#
#   ${PAPERSIZE} Paper size ("letter", "a4", "a3", etc.)
#   ${ORIENT}    Page orientation, "portrait"  (default) or " landscape" 
#   ${MACROS}    List of latex macro files
#   ${PARTS}     List of other latex included files (sections, tables, etc.)
#   ${FIGURES}   List of EPS figures
#   ${FIGDIRS}   List of figure directories that need "make"
#   ${BIBS}      List of bibliography files 
#   ${EXDIR}     WWW subdirectory where to put a copy of the derived files
#   ${CMPDIR}    Directory containing the previous version, for {diff}.
#
#   ${SUBMITNAME}    Name for the submission archive, minus extension
#   ${SUBMITFORMAT}  Type of submission archive (`tgz' or `zip')
#   ${SUBMITFILES}   List of files to include in submission archive
# 
# All file names must be relative to the current directory. Rather
# than using TEXINPUTS mechanism for macros in other directories, the
# user should create local copies or symbolic links to them. This will
# allow the "archive" action to copy them with useful names.
# 
# Also assumes that there is a ${STOLFIHOME} enviromment variable
# that contains bin/run-latex, bin/run-pdflatex, etc..
# 
# We do not include ${STOLFIHOME}/tex/inputs in the search path.
# Users should consider creating symlinks to macro files for better archival.

ifneq "/${JOBNAME}" "/"
ifneq "/${FORMAT}" "/"
ifneq "/${STOLFIHOME}" "/"

.SUFFIXES:

.PHONY: \
  all default-target \
  make-pdf \
  show-ps show-pdf \
  export-derived \
  archive \
  submit

default-target: make-${FORMAT} show-${FORMAT}

SET_TEX_PATHS := \
  export TEXINPUTS=".::"

ifeq "/${FORMAT}" "/ps"

# ----------------------------------------------------------------------
# Creating PDF output from ".tex" via ".dvi" and Postscript

# We do not include ${STOLFIHOME}/tex/inputs in the search path.
# Users should consider creating symlinks to macro files for better archival.

OUT_DOCS := ${JOBNAME}.pdf ${JOBNAME}.ps ${JOBNAME}.dvi

RUN_LATEX := \
  export PATH="$${STOLFIHOME}/bin:$${PATH}" ; \
    ${STOLFIHOME}/bin/run-latex

${JOBNAME}.dvi: ${JOBNAME}.tex ${BIBS} ${PARTS} ${MACROS} ${FIGDIRS} ${FIGURES}
	${SET_TEX_PATHS}; ${RUN_LATEX} ${JOBNAME}.tex
	tex-list-eps-figs ${JOBNAME} | sort | uniq > ${JOBNAME}.figs

make-pdf: ${JOBNAME}.pdf show-pdf export-pdf

COMMA := ,
ifeq "/${PAPERSIZE}" "/"
  PAPERSIZE_OPTS := 
else
ifeq "/${findstring ${COMMA},${PAPERSIZE}}" "/,"
  PAPERSIZE_OPTS := -T ${PAPERSIZE}
else
  PAPERSIZE_OPTS := -t ${PAPERSIZE}
endif
endif

ifeq "/${ORIENT}" "/"
  ORIENT_OPTS := 
else
  ORIENT_OPTS := -t ${ORIENT}
endif

${JOBNAME}.ps: ${JOBNAME}.dvi ${FIGURES}
	${SET_TEX_PATHS} ; \
           dvips -o ${JOBNAME}.ps \
             -D600 \
             ${PAPERSIZE_OPTS} \
             ${ORIENT_OPTS} \
             ${JOBNAME}

${JOBNAME}.pdf: ${JOBNAME}.ps
	${SET_TEX_PATHS}; ps2pdf ${JOBNAME}.ps ${JOBNAME}.pdf

ifneq "/${EXDIR}" "/"
  export-derived: ${JOBNAME}.ps ${JOBNAME}.pdf
	gzip ${JOBNAME}.ps && cp -pv ${JOBNAME}.ps.gz ${EXDIR}/ && gunzip ${JOBNAME}.ps
	cp -pv ${JOBNAME}.pdf ${EXDIR}/
else
  export-derived: 
endif  
       
endif

ifeq "/${FORMAT}" "/pdf"

# ----------------------------------------------------------------------
# Creating PDF output directly from ".tex"

OUT_DOCS := ${JOBNAME}.pdf

RUN_PDFLATEX := \
  export PATH="$${STOLFIHOME}/bin:$${PATH}" ; \
    ${STOLFIHOME}/bin/run-pdflatex

${JOBNAME}.pdf: ${JOBNAME}.tex ${BIBS} ${PARTS} ${MACROS} ${FIGDIRS} ${FIGURES}
	${SET_TEX_PATHS}; ${RUN_PDFLATEX} ${JOBNAME}.tex
	tex-list-eps-figs ${JOBNAME} | sort | uniq > ${JOBNAME}.figs

make-pdf: ${JOBNAME}.pdf show-pdf export-derived

ifneq "/${EXDIR}" "/"
export-derived: ${JOBNAME}.pdf
	cp -pv ${JOBNAME}.pdf ${EXDIR}/
else
export-derived: 

endif  

endif

# ----------------------------------------------------------------------
# Displaying the PS or PDF output:
# These definitions are needed independently of chosen format:
 
SHOWPS := evince

show-ps: ${JOBNAME}.ps
	${SHOWPS} ${JOBNAME}.ps
        
# SHOWPDF := acroread
# SHOWPDF := gv
SHOWPDF := okular

show-pdf: ${JOBNAME}.pdf
	${SHOWPDF} ${JOBNAME}.pdf

# ----------------------------------------------------------------------
# Standard local symbolic links

run-latex:
	ln -s ${STOLFIHOME}/bin/run-latex
        
GENERIC-LATEX-PAPER.make:
	ln -s ${STOLFIHOME}/papers/GENERIC-LATEX-PAPER.make

stolfi.bib:
	ln -s ${STOLFIHOME}/EXPORT/bibliography/stolfi.bib

others.bib:
	ln -s ${STOLFIHOME}/EXPORT/bibliography/others.bib

students.bib:
	ln -s ${STOLFIHOME}/EXPORT/bibliography/students.bib

# ----------------------------------------------------------------------
# make archive - Create ".tgz" and ".zip" snapshots of final paper and
# all necessary files, including the targets of symbolic links.
# If the ${EXDIR} variable is defined, the archives are placed therein.
# 
# This script assumes that archiving the ${FIGDIRS} directories
# will also save all ${FIGURES} files.

ARCHIVE_DATE := ${shell date '+%Y-%m-%d-%H%M%S'}

ifneq "/${EXDIR}" "/"
   ARCHIVE_DIR := ${EXDIR}
else
   ARCHIVE_DIR := .
endif        

TGZ_ARCHIVE := ${ARCHIVE_DIR}/${JOBNAME}-${ARCHIVE_DATE}.tgz
ZIP_ARCHIVE := ${ARCHIVE_DIR}/${JOBNAME}-${ARCHIVE_DATE}.zip

ARCHIVES := \
  ${TGZ_ARCHIVE} \
  ${ZIP_ARCHIVE}

ARCHIVEFILES := \
  ${OUT_DOCS} \
  ${JOBNAME}.tex \
  ${PARTS} \
  ${BIBS} \
  ${MACROS} \
  ${JOBNAME}.make \
  ${FIGDIRS} \
  ${JOBNAME}.bbl \
  run-latex \
  run-pdflatex \
  GENERIC-LATEX-PAPER.make

# A symbolic link to the current directory with a suitable name:

ARCHIVE_PSEUDO_DIR := ${ARCHIVE_DATE}

# The purpose of this link is to ensure that all items in the tar archive
# start with "${ARCHIVE_DATE}/".  This helps avoid confusion when
# the archive is unpacked.

archive: ${ARCHIVES}

${ARCHIVE_PSEUDO_DIR}:
	ln -s . ${ARCHIVE_PSEUDO_DIR}

FILE_NAMES_IN_ARCHIVE := ${addprefix ${ARCHIVE_PSEUDO_DIR}/, ${ARCHIVEFILES}}

${TGZ_ARCHIVE}: ${ARCHIVE_PSEUDO_DIR} ${ARCHIVEFILES}
	tar -cvhzf ${TGZ_ARCHIVE} ${FILE_NAMES_IN_ARCHIVE}

${ZIP_ARCHIVE}: ${ARCHIVE_PSEUDO_DIR} ${ARCHIVEFILES}
	zip -r ${ZIP_ARCHIVE} ${FILE_NAMES_IN_ARCHIVE}

########################################################################
# Submission archive section
# Client must define ${SUBMITNAME} ${SUBMITFORMAT}
#
# `make submit' create an archive with name 
# ${SUBMITNAME}.${SUBMITFORMAT} with  the files listed in
# ${SUBMITFILES}

ifneq "/${SUBMITNAME}" "/"
ifneq "/${SUBMITFORMAT}" "/"

SUBMARCHIVE := ${SUBMITNAME}.${SUBMITFORMAT}

${SUBMITNAME}.tgz: ${SUBMITFILES}
	tar -cvhzf ${SUBMITNAME}.tgz ${SUBMITFILES}

${SUBMITNAME}.zip: ${SUBMITFILES}
	zip -r ${SUBMITNAME} ${SUBMITFILES}

endif        
endif        
# end of ${SUBMITNAME} ${SUBMITFORMAT} section
########################################################################

######################################################################
# Compare with previous version:
# Client must define ${CMPDIR}

ifneq "/${CMPDIR}" "/"

DIFFS := ${subst .tex,.diff, ${JOBNAME}.tex ${PARTS} ${MACROS}} 

compare: ${DIFFS}

%.diff: %.tex ${CMPDIR}/$%.tex
	diff -Bb $*.tex ${CMPDIR}/$*.tex \
          | prettify-diff-output \
          > $*.diff

else
compare: 
	@ echo "No previous version to compare against"
endif
# end of ${CMPDIR} section
######################################################################

# ----------------------------------------------------------------------
# `make clean' removes the derived files, except the .bbl (which is
# sometimes a soruce file).

clean::
	rm -f ${JOBNAME}.{toc,lot,lof,log,blg,aux}{,~~}
	rm -f ${JOBNAME}.{errs,dvi,ps,pdf}

endif
endif
endif
# end of ${JOBNAME} ${FORMAT} ${STOLFIHOME} section
########################################################################
